<?php

/*
  Plugin Name: SmartRoute Direct Post Payment - WooCommerce Gateway
  Plugin URI: http://www.stspayone.com/
  Description: Extends WooCommerce by Adding the SmartRoute Direct Post Payment Gateway.
  Version: 1.0.4
  Author: STS PayOne
  Author URI: http://www.stspayone.com/
 */

// Include our Gateway Class and register Payment Gateway with WooCommerce
add_action('plugins_loaded', 'smartroute_direct_post_init', 1);
add_action('plugins_loaded', 'directpost_load_textdomain', 0);
add_action('plugins_loaded', 'schedule_directpost_inquiry_operations', 2);

function smartroute_direct_post_init() {
    // If the parent WC_Payment_Gateway class doesn't exist
    // it means WooCommerce is not installed on the site
    // so do nothing
    if (!class_exists('WC_Payment_Gateway'))
        return;

    //directpost_load_textdomain();
    // If we made it this far, then include our Gateway Class
    include_once( 'woocommerce-smartroute-direct-post.php' );

    // Now that we have successfully included our class,
    // Lets add it too WooCommerce
    add_filter('woocommerce_payment_gateways', 'add_smartroute_direct_post_gateway');

    function add_smartroute_direct_post_gateway($methods) {
        $methods[] = 'SmartRoute_Direct_Post';
        return $methods;
    }

}

// Add custom action links
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'smartroute_direct_post_action_links');

function smartroute_direct_post_action_links($links) {
    $plugin_links = array(
        '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout') . '">' . __('Settings', 'smartroute-direct-post') . '</a>',
    );

    // Merge our new link with the default ones
    return array_merge($plugin_links, $links);
}

add_action('plugins_loaded', 'smartroute_direct_post_run');

/**
 * Run the plugin.
 */
function smartroute_direct_post_run() {
    require_once('functions.php' );
}

add_action('init', 'add_direct_post_endpoint');

function add_direct_post_endpoint() {
    add_rewrite_endpoint('handleDpResponse', EP_ROOT | EP_PAGES);
}

add_action('woocommerce_before_checkout_form_cart_notices', 'include_payment_details', 10, 5);

function include_payment_details() {
    global $wp_query;

    if (isset($wp_query->query['handleDpResponse']) &&
            (!isset($_SESSION['smartRouteDirectPostEntered']) || !$_SESSION['smartRouteDirectPostEntered']) ) {
        load_template(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . SmartRoute_Direct_Post::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'woocommerce-smartroute-direct-post-response.php');
    }

    return true;
}

/*
This is how t he above function looked like when STS received this plug in
The below method replaced each of the searchbox, breadcrumbs,and reattempt payment views with the payment details
The new function includes the payment details without touching the original page structure

add_filter('wc_get_template', 'custom_direct_post_endpoint', 10, 5);

function custom_direct_post_endpoint($located, $template_name, $args, $template_path, $default_path) {
    global $wp_query;

    if (isset($wp_query->query['handleDpResponse']) &&
            (!isset($_SESSION['smartRouteDirectPostEntered']) || !$_SESSION['smartRouteDirectPostEntered']) ) {

echo "<script>console.log('" . $template_name . "')</script>";

        $located = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . SmartRoute_Direct_Post::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'woocommerce-smartroute-direct-post-response.php';
    }

    return $located;
}
*/

function directpost_load_textdomain() {
    load_plugin_textdomain('woocommerce-smartroute-direct-post-gateway', false, dirname(plugin_basename(__FILE__)) . '/i18n/languages/');
}

function schedule_directpost_inquiry_operations() {
    add_filter('cron_schedules', 'callback_add_inquiry_directpost_schedule');

    add_action(
            'smartroute_directpost_inquiry_cron_hook', 'smartroute_directpost_inquiry_hook'
    );

    if (!wp_next_scheduled('smartroute_directpost_inquiry_cron_hook')) {
        wp_schedule_event(
                time(), 'directpost-inquiry-interval', 'smartroute_directpost_inquiry_cron_hook'
        );
    }

    return;
}

function callback_add_inquiry_directpost_schedule($schedules) {
    $schedules['directpost-inquiry-interval'] = array(
        'interval' => 5 * 60, //every 10 minutes
        'display' => __('Direct Post Inquiry Interval (5 minutes)', 'woocommerce-smartroute-direct-post-gateway'),
    );
    return $schedules;
}

function smartroute_directpost_inquiry_hook() {
    //error_log("scheduled job " . __METHOD__ . ' is called...');

    $args = array(        
        'status' => array('wc-pending', 'wc-processing','wc-cancelled'),        
        'payment_method' => 'smartroute_direct_post',
        'created_via' => 'checkout',
        'type' => 'shop_order',
        'date_before' => gmdate('Y-m-d H:i:s', time() - 5 * 60),
        'limit' => -1,
    );
    $orders = wc_get_orders($args);

    $gateway = new Smartroute_Direct_Post();    
    foreach($orders as $order){
        if($order->get_transaction_id()){
            $gateway->log('calling inquiry for order trans ID#'.$order->get_id());
            $gateway->inquirySingleOrder($order);
        }
    }    
}

