<?php

/* SmartRoute Direct Post Payment Gateway Class */
include 'woocommerce-smartroute-direct-post-helper.php';

class Smartroute_Direct_Post extends WC_Payment_Gateway {

    public static $PLUGIN_FOLDER_NAME = 'woocommerce-smartroute-direct-post-gateway';
    private static $log;
    public static $invalidSecureHashInquiryURL;

    // Setup our Gateway's id, description and other values
    function __construct() {
        // The global ID for this Payment method
        $this->id = "smartroute_direct_post";

        // The Title shown on the top of the Payment Gateways Page next to all the other Payment Gateways
        $this->method_title = __("SmartRoute Direct Post Payment", 'woocommerce-smartroute-direct-post-gateway');

        $this->supports = array('default_credit_card_form');

        $this->supports[] = 'refunds';

        // The description for this Payment Gateway, shown on the actual Payment options page on the backend
        $this->method_description = __("SmartRoute Direct Post Payment Gateway Plug-in for WooCommerce", 'woocommerce-smartroute-direct-post-gateway');

        // The title to be used for the vertical tabs that can be ordered top to bottom
        $this->title = __("SmartRoute Direct Post Payment", 'woocommerce-smartroute-direct-post-gateway');

        // If you want to show an image next to the gateway's name on the frontend, enter a URL to an image.
        $this->icon = WP_PLUGIN_URL . DIRECTORY_SEPARATOR . Smartroute_Direct_Post::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'SmartRoute.png';


        // Bool. Can be set to true if you want payment fields to show on the checkout 
        // if doing a direct integration, which we are doing in this case
        $this->has_fields = true;

        // This basically defines your settings which are then loaded with init_settings()
        $this->init_form_fields();

        // After init_settings() is called, you can get the settings and load them into variables, e.g:
        // $this->title = $this->get_option( 'title' );
        $this->init_settings();

        // Turn these settings into variables we can use
        foreach ($this->settings as $setting_key => $value) {
            $this->$setting_key = $value;
        }

        // Lets check for SSL
        add_action('admin_notices', array($this, 'do_ssl_check'));


        // Save settings
        if (is_admin()) {
            // Versions over 2.0
            // Save our administration options. Since we are not going to be doing anything special
            // we have not defined 'process_admin_options' in this class so the method in the parent
            // class will be used instead
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        }

        Smartroute_Direct_Post::$invalidSecureHashInquiryURL = $this->InquiryURL;
    }

    // End __construct()
    // Build the administration fields for this specific Gateway
    public function init_form_fields() {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable / Disable', 'woocommerce-smartroute-direct-post-gateway'),
                'label' => __('Enable this payment gateway', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'checkbox',
                'default' => 'no',
            ),
            'title' => array(
                'title' => __('Title', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'text',
                'desc_tip' => __('Payment title the customer will see during the checkout process.', 'woocommerce-smartroute-direct-post-gateway'),
                'default' => __('SmartRoute Direct Post Payment', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'description' => array(
                'title' => __('Description', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'textarea',
                'desc_tip' => __('Payment description the customer will see during the checkout process.', 'woocommerce-smartroute-direct-post-gateway'),
                'default' => __('Pay securely using your Credit card or SADAD account', 'woocommerce-smartroute-direct-post-gateway') . '.',
                'css' => 'max-width:350px;'
            ),
            'MerchantID' => array(
                'title' => __('Merchant ID', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'text',
                'desc_tip' => __('This is the Merchant ID.', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'auth_token' => array(
                'title' => __('Authentication Token', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'text',
                'desc_tip' => __('This is the Authentication Token provided by SmartRoute Direct Post Payment for the merchant.', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'ItemID' => array(
                'title' => __('Item ID', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'text',
                'desc_tip' => __('This is the custom Item ID for the merchant.', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'RedirectPaymentURL' => array(
                'title' => __('Redirect Payment URL', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'text',
                'desc_tip' => __('This is the payment URL for smartroute direct-post.', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'RefundURL' => array(
                'title' => __('Refund URL', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'text',
                'desc_tip' => __('This is the refund URL for smartroute direct-post.', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'InquiryURL' => array(
                'title' => __('Inquiry URL', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'text',
                'desc_tip' => __('This is the inquiry URL for smartroute direct-post.', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'MethodType' => array(
                'title' => __('Method Type', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'multiselect',
                'options' => array(
                    'sadad' => 'Sadad',
                    'creditcard' => 'Credit Card',
                ),
                'desc_tip' => __('This is for enabling sadad and/or credit card options for smartroute direct-post.', 'woocommerce-smartroute-direct-post-gateway'),
            ),
            'enableLog' => array(
                'title' => __('Enable Logs', 'woocommerce-smartroute-direct-post-gateway'),
                'label' => __('Enable logs for this payment gateway', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'checkbox',
                'default' => 'no',
            ),
        );
    }

    // Submit payment and handle response
    public function process_payment($order_id) {

        // Get this Order's information so that we know
        // who to charge and how much
        $order = new WC_Order($order_id);

        // Decide which URL to post to
        $currencies_file = parse_ini_file("currencies.ini");

        // This is where the fun stuff begins
        $data = array();
        $data['MessageID'] = 1;
        $data['TransactionID'] = (int) (microtime(true) * 1000);
        $data['MerchantID'] = trim($this->MerchantID);
        
	$this->log('Processing payment for order #' . $order_id . ', and transaction ID ' . $data['TransactionID'] . '.');

        $old_wc = version_compare(WC_VERSION, '3.0', '<');
        $order_total = $old_wc ? $order->order_total : $order->get_total();
        $data['Amount'] = str_replace('.', '', $order_total);
        $data['CurrencyISOCode'] = $currencies_file[get_woocommerce_currency()];
        if (strpos(get_locale(), '_') !== false) {
            $data['Language'] = substr(get_locale(), 0, strpos(get_locale(), '_'));
        } else {
            $data['Language'] = get_locale();
        }
        if (strpos(wc_get_page_permalink('checkout'), '?') !== false) {
            $data['ResponseBackURL'] = wc_get_page_permalink('checkout') . '&handleDpResponse';
        } else {
            $data['ResponseBackURL'] = wc_get_page_permalink('checkout') . '?handleDpResponse';
        }
        $data['PaymentMethod'] = $_REQUEST['SrdpPaymentMethod'];

        //set transaction ID inside order
        $order->set_transaction_id($data['TransactionID']);
        $order->save();

        if (trim($this->ItemID) != '') {
            $data['ItemID'] = trim($this->ItemID);
        }
        $data['SecureHash'] = $this->generateDirectPostSecureHash($data, $this->auth_token);

        $action = trim($this->RedirectPaymentURL);

        $error_message = 'Please fill the required fields';

        if ($data['PaymentMethod'] == 1) {
            $data['CardNumber'] = $_REQUEST['SrdpCardNumber'];
            $data['ExpiryDateMonth'] = $_REQUEST['SrdpExpiryDateMonth'];
            $data['ExpiryDateYear'] = $_REQUEST['SrdpExpiryDateYear'];
            $data['SecurityCode'] = $_REQUEST['SrdpSecurityCode'];
            if (isset($_REQUEST['SrdpCardHolderName'])) {
                $data['CardHolderName'] = $_REQUEST['SrdpCardHolderName'];
            }

            // check if not empty
            if ($data['CardNumber'] == '' || $data['ExpiryDateMonth'] == '' || $data['ExpiryDateYear'] == '' || $data['SecurityCode'] == '' || $data['CardHolderName'] == '') {
                wc_add_notice($error_message, 'error');
                $this->log('Transaction failed: ' . $error_message);
                return;
            }
            // check if numeric
            if (!is_numeric($data['CardNumber'])) {
                wc_add_notice('Invalid card number', 'error');
                $this->log('Transaction failed: Invalid card number');
                return;
            }

            if (!is_numeric($data['SecurityCode'])) {
                wc_add_notice('Invalid cvv', 'error');
                $this->log('Transaction failed: Invalid cvv');
                return;
            }
        } else if ($data['PaymentMethod'] == 2) {
            $data['SadadOlpId'] = $_REQUEST['SrdpSadadOlpId'];
            if ($data['SadadOlpId'] == '') {
                wc_add_notice($error_message, 'error');
                $this->log('Transaction failed: ' . $error_message);
                return;
            }
        }

        //$order->add_order_note($order->order_key);
        // Send this payload to SmartRoute Direct Post Payment for processing
        $query_string = "";
        $first_param = true;
        foreach ($data as $key => $value) {
            if ($first_param) {
                $query_string .= $key . '=' . $value;
                $first_param = false;
            } else {
                $query_string .= '&' . $key . '=' . $value;
            }
        }

        session_start();
        $_SESSION["auth_token"] = $this->auth_token;
        $_SESSION["order_id"] = $order_id;

	$this->log('Transaction sent to Smartroute');
        return array(
            'result' => 'success',
            'redirect' => $action,
            'params' => $query_string,
            'redirect_type' => 'post'
        );
    }

    // Validate fields
    public function validate_fields() {
        return true;
    }

    // Check if we are forcing SSL on checkout pages
    // Custom function not required by the Gateway
    public function do_ssl_check() {

        if ($this->enabled == "yes") {
            if (get_option('woocommerce_force_ssl_checkout') == "no") {
                echo "<div class=\"error\"><p>" . sprintf(__("<strong>%s</strong> is enabled and WooCommerce is not forcing the SSL certificate on your checkout page. Please ensure that you have a valid SSL certificate and that you are <a href=\"%s\">forcing the checkout pages to be secured.</a>"), $this->method_title, admin_url('admin.php?page=wc-settings&tab=checkout')) . "</p></div>";
            }
        }
    }

    public function process_refund($order_id, $amount = null, $reason = '') {

        $order = wc_get_order($order_id);
        if (!$order) {
            return false;
        }

        $this->log("Beginning refund for order $order_id for the amount of {$amount}", 'info');

        $currencies_file = parse_ini_file("currencies.ini");

        $url = trim($this->RefundURL);

        if ($order->post_status == 'wc-failed') {
            return new WP_Error('smartroute_error', __('Order is not suitable for refund.', 'woocommerce-smartroute-direct-post-gateway'));
        }

        // check if the amount is empty
        if ($amount == null || $amount == "null" || $amount == '' || $amount == 0) {
            return new WP_Error('smartroute_error', __('The refund amount should not be empyt or 0.', 'woocommerce-smartroute-direct-post-gateway'));
        }

        // check if the amount less than or equal zero
        if ($amount < 0) {
            return new WP_Error('smartroute_error', __('Invalid refund amount.', 'woocommerce-smartroute-direct-post-gateway'));
        }

        //prepare the request parameters
        $transactionID = "wcref" . time() . rand(1000, 9999);
        $fields = array();
        $fields['MessageID'] = 4;
        $fields['MerchantID'] = trim($this->MerchantID);
        $fields['TransactionID'] = $transactionID;
        $fields['OriginalTransactionID'] = $order->get_transaction_id();
        $fields['CurrencyISOCode'] = $currencies_file[get_woocommerce_currency()];
        $fields['Amount'] = str_replace('.', '', $amount);
        $fields['SecureHash'] = $this->generateDirectPostSecureHash($fields);

        $this->log("Directpost refund order#{$order_id} parameters: " . json_encode($fields), 'debug');

        $response = wp_remote_post($url, array(
            'method' => 'POST',
            'body' => http_build_query($fields),
            'timeout' => 90,
            'sslverify' => false,
        ));

        // Retrieve the body's resopnse if no errors found
        $response_body = wp_remote_retrieve_body($response);
        $this->log("Directpost refund order#{$order_id} response: " . $response_body, 'debug');
        // Parse the response into something we can read
        $result = array();
        foreach (explode("&", $response_body) as $parameter) {
            $equalsIndex = strpos($parameter, "=");
            $name = substr($parameter, 0, $equalsIndex);
            $value = substr($parameter, $equalsIndex + 1);

            $result[$name] = $value;
        }

        $res_data = array();

        //get required params from response
        if (isset($result['Response.OriginalTransactionID'])) {
            $res_data['Response.OriginalTransactionID'] = $this->checkIfNull($result['Response.OriginalTransactionID']);
        }

        if (isset($result['Response.StatusCode'])) {
            $res_data['Response.StatusCode'] = $this->checkIfNull($result['Response.StatusCode']);
        }

        if (isset($result['Response.StatusDescription'])) {
            $res_data['Response.StatusDescription'] = $this->checkIfNull($result['Response.StatusDescription']);
        }

        if (isset($result['Response.Amount'])) {
            $res_data['Response.Amount'] = $this->checkIfNull($result['Response.Amount']);
        }

        if (isset($result['Response.CurrencyISOCode'])) {
            $res_data['Response.CurrencyISOCode'] = $this->checkIfNull($result['Response.CurrencyISOCode']);
        }

        if (isset($result['Response.MerchantID'])) {
            $res_data['Response.MerchantID'] = $this->checkIfNull($result['Response.MerchantID']);
        }

        if (isset($result['Response.TransactionID'])) {
            $res_data['Response.TransactionID'] = $this->checkIfNull($result['Response.TransactionID']);
        }

        if (isset($result['Response.MessageID'])) {
            $res_data['Response.MessageID'] = $this->checkIfNull($result['Response.MessageID']);
        }

        //get optional params from response
        if (isset($result['Response.RRN'])) {
            $res_data['Response.RRN'] = $this->checkIfNull($result['Response.RRN']);
        }

        // get the secure hash
        $success = false;
        $is_inq = false;
        $res_secure_hash = $this->checkIfNull($result['Response.SecureHash']);
        $newSecureHash = $this->generateDirectPostSecureHash($res_data);

        $status = (isset($res_data['Response.StatusCode']) AND ! empty($res_data['Response.StatusCode'])) ? $res_data['Response.StatusCode'] : NULL;

        //test the securehash
        if ($status != NULL AND $newSecureHash == $res_secure_hash) {
            if ($res_data['Response.StatusCode'] == "00000") {
                $success = true;
                if ($order->post_status == 'wc-pending') {
                    $order->update_status('wc-completed', 'order_note');
                }

                // status code not success and not abnormal error
            } else if ($res_data['Response.StatusCode'] != "00019") {
                $order->update_status('wc-failed', 'order_note');
                return new WP_Error('smartroute_error', __('Order is not suitable for refund.', 'woocommerce-smartroute-direct-post-gateway'));
            }
        } else {
            //Inquiry Message
            $is_inq = true;
            $this->log('Directpost refund inquiry to be called, order# ', 'debug');

            $inq_url = trim($this->InquiryURL);

            $fields = array();
            $fields['MessageID'] = 2;
            $fields['MerchantID'] = trim($this->MerchantID);
            $fields['OriginalTransactionID'] = $transactionID;//$res_data['Response.TransactionID']wcrf;
            $fields['SecureHash'] = $this->generateDirectPostSecureHash($fields);

            $this->log('Directpost refund inquiry parameters: ' . json_encode($fields), 'debug');
            $inq_response = wp_remote_post($inq_url, array(
                'method' => 'POST',
                'body' => http_build_query($fields),
                'timeout' => 90,
                'sslverify' => false,
            ));

            // Retrieve the body's resopnse if no errors found
            $inq_response_body = wp_remote_retrieve_body($inq_response);
            $this->log('Directpost refund inquiry response: ' . $inq_response_body, 'debug');
            // Parse the response into something we can read
            $inq_resp = array();
            foreach (explode("&", $inq_response_body) as $parameter) {
                $equalsIndex = strpos($parameter, "=");
                $name = substr($parameter, 0, $equalsIndex);
                $value = substr($parameter, $equalsIndex + 1);

                $inq_resp[$name] = $value;
            }
            $inq_res_data = array();
            //get required params from response
            if (isset($inq_resp['Response.MessageStatus'])) {
                $inq_res_data['Response.MessageStatus'] = $this->checkIfNull($inq_resp['Response.MessageStatus']);
            }

            if (isset($inq_resp['Response.StatusCode'])) {
                $inq_res_data['Response.StatusCode'] = $this->checkIfNull($inq_resp['Response.StatusCode']);
            }

            if (isset($inq_resp['Response.StatusDescription'])) {
                $inq_res_data['Response.StatusDescription'] = $this->checkIfNull($inq_resp['Response.StatusDescription']);
            }

            if (isset($inq_resp['Response.Amount'])) {
                $inq_res_data['Response.Amount'] = $this->checkIfNull($inq_resp['Response.Amount']);
            }

            if (isset($inq_resp['Response.CurrencyISOCode'])) {
                $inq_res_data['Response.CurrencyISOCode'] = $this->checkIfNull($inq_resp['Response.CurrencyISOCode']);
            }

            if (isset($inq_resp['Response.MerchantID'])) {
                $inq_res_data['Response.MerchantID'] = $this->checkIfNull($inq_resp['Response.MerchantID']);
            }

            if (isset($inq_resp['Response.TransactionID'])) {
                $inq_res_data['Response.TransactionID'] = $this->checkIfNull($inq_resp['Response.TransactionID']);
            }

            if (isset($inq_resp['Response.MessageID'])) {
                $inq_res_data['Response.MessageID'] = $this->checkIfNull($inq_resp['Response.MessageID']);
            }

            if (isset($inq_resp['Response.ReversalStatus'])) {
                $inq_res_data['Response.ReversalStatus'] = $this->checkIfNull($inq_resp['Response.ReversalStatus']);
            }

            //get optional params from response
            if (isset($inq_resp['Response.GatewayStatusCode'])) {
                $inq_res_data['Response.GatewayStatusCode'] = $this->checkIfNull($inq_resp['Response.GatewayStatusCode']);
            }

            if (isset($inq_resp['Response.GatewayStatusDescription'])) {
                $inq_res_data['Response.GatewayStatusDescription'] = $this->checkIfNull($inq_resp['Response.GatewayStatusDescription']);
            }

            if (isset($inq_resp['Response.GatewayName'])) {
                $inq_res_data['Response.GatewayName'] = $this->checkIfNull($inq_resp['Response.GatewayName']);
            }

            if (isset($inq_resp['Response.RRN'])) {
                $inq_res_data['Response.RRN'] = $this->checkIfNull($inq_resp['Response.RRN']);
            }

            if (isset($inq_resp['Response.ApprovalCode'])) {
                $inq_res_data['Response.ApprovalCode'] = $this->checkIfNull($inq_resp['Response.ApprovalCode']);
            }

            if (isset($inq_resp['Response.CardExpiryDate'])) {
                $inq_res_data['Response.CardExpiryDate'] = $this->checkIfNull($inq_resp['Response.CardExpiryDate']);
            }

            if (isset($inq_resp['Response.CardHolderName'])) {
                $inq_res_data['Response.CardHolderName'] = $this->checkIfNull($inq_resp['Response.CardHolderName']);
            }

            if (isset($inq_resp['Response.CardNumber'])) {
                $inq_res_data['Response.CardNumber'] = $this->checkIfNull($inq_resp['Response.CardNumber']);
            }

            // get the secure hash
            $res_secure_hash = $this->checkIfNull($inq_resp['Response.SecureHash']);
            $newSecureHash = $this->generateDirectPostSecureHash($inq_res_data);

            if ($newSecureHash == $res_secure_hash) {
                if ($inq_res_data['Response.StatusCode'] == "00000") {
                    $success = true;
                    if ($order->post_status == 'wc-pending') {
                        $order->update_status('wc-completed', 'order_note');
                    }
                    // status code not success and not abnormal error	
                } else if ($res_data['Response.StatusCode'] != "00019") {
                    $order->update_status('wc-failed', 'order_note');
                    return new WP_Error('smartroute_error', __('Order is not suitable for refund.', 'woocommerce-smartroute-direct-post-gateway'));
                }
            } else {
                // do nothing
            }
        }

        if ($success) {
            if ($is_inq) {
                $refund_message = sprintf(__('Refunded %1$s - Refund ID: %2$s - Reason: %3$s', 'woocommerce-smartroute-direct-post-gateway'), $amount, $inq_res_data['Response.TransactionID'], $reason);
            } else {
                $refund_message = sprintf(__('Refunded %1$s - Refund ID: %2$s - Reason: %3$s', 'woocommerce-smartroute-direct-post-gateway'), $amount, $res_data['Response.TransactionID'], $reason);
            }
            $order->add_order_note($refund_message);
            $this->log('Success: ' . html_entity_decode(strip_tags($refund_message)));
            return true;
        } else {
            $this->log('Error: ' . $res_data['Response.StatusDescription']);
            return new WP_Error('smartroute_error', __('There was a problem connecting to the payment gateway.', 'woocommerce-smartroute-direct-post-gateway'));
        }
    }

    /**
     * Format prices.
     * @param  float|int $price
     * @param  WC_Order $order
     * @return string
     */
    protected function number_format($price, $order) {
        $decimals = 2;

        if (!$this->currency_has_decimals($order->get_currency())) {
            $decimals = 0;
        }

        return number_format($price, $decimals, '.', '');
    }

    /**
     * Check if currency has decimals.
     * @param  string $currency
     * @return bool
     */
    protected function currency_has_decimals($currency) {
        if (in_array($currency, array('HUF', 'JPY', 'TWD'))) {
            return false;
        }

        return true;
    }

    public function credit_card_form($args = array(), $fields = array()) {
        $this->form();
    }

    public function form() {
        $months = array("01", "02", "03", "04", "05", "06", "07", "08", "09", "10", "11", "12");

        echo '<link rel="stylesheet" href="' . WP_PLUGIN_URL . DIRECTORY_SEPARATOR . Smartroute_Direct_Post::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'bootstrap-customized.css">';

        echo '<div class = "panel-group">';
        echo '<div class = "panel panel-default">';
        echo '<div class="panel-default">';
        echo '<div class="panel-body">';
        if (in_array('creditcard', $this->MethodType)) {
            echo '<div class="col-sm-12">';
            echo '<div class="col-sm-12 noPadding">';
            echo '<ul class="wc_payment_methods payment_methods methods"><li><input type="radio" name="SrdpPaymentMethod" autocomplete="off" value="1" checked="checked" id="SrdpPaymentMethod-credit" /> <label for="SrdpPaymentMethod-credit" class="control-label">  ' . __('Credit Card', 'woocommerce-smartroute-direct-post-gateway') . '</label></li></ul>';
            echo '</div>';
            echo '';
            echo '<div class="col-sm-12">';
            echo '<label class="control-label">' . __('Card Number', 'woocommerce-smartroute-direct-post-gateway') . ' <span style="color:red">*</span> </label>';
            echo '</div>';
            echo '<div data-payment-method="1">';
            echo '<div class="col-sm-12">';
            echo '<input type="text" name="SrdpCardNumber" autocomplete="off" data-validate="required|creditcard" value="" class="form-control requiredField validate-required " maxlength="23"/>';
            echo '</div>';
            echo '';
            echo '<div class="col-sm-12"><br/></div>';
            echo '';
            echo '<div class="col-sm-12">';
            echo '<label class="control-label">' . __('Expiry Date', 'woocommerce-smartroute-direct-post-gateway') . ' <span style="color:red">*</span> </label>';
            echo '</div>';
            echo '<div class="col-sm-12 form-row">';
            echo '<div class="col-sm-6 col-xs-6 noPadding">';
            echo '<select name="SrdpExpiryDateMonth" class="form-control requiredField validate-required" data-validate="required">';
            echo '<option value="" selected="selected">MM</option>';


            for ($i = 1; $i <= 9; $i++) {
                echo '<option value="' . $months[$i - 1] . '" >' . '0' . $i . '</option>';
            }
            for ($i = 10; $i <= 12; $i++) {
                echo '<option value="' . $months[$i - 1] . '" >' . $i . '</option>';
            }

            echo '</select>';
            echo '</div>';
            echo '<div class="col-sm-6 col-xs-6 noPadding">';
            echo '<select name="SrdpExpiryDateYear" style="width: 92%;" data-validate="required" class="form-control requiredField validate-required">';
            echo '<option value="" selected="selected">YYYY</option>';

            for ($i = date("Y"); $i <= date("Y") + 10; $i++) {
                echo '<option value="' . substr($i, 2, 4) . '">' . $i . '</option>';
            }

            echo '</select>';
            echo '</div>';
            echo '</div>';
            echo '';
            //echo '<div class="col-sm-12"><br/></div>';
            echo '<div class="clearfix">&nbsp;</div>';
            echo '';
            echo '<div class="col-sm-12">';
            echo '<label class="control-label">' . __('Security Code', 'woocommerce-smartroute-direct-post-gateway') . ' <span style="color:red">*</span> </label>';
            echo '</div>';
            echo '<div class="col-sm-12">';
            echo '<input type="password" name="SrdpSecurityCode" id="SrdpSecurityCode" value="" data-validate="required|cvv" autocomplete="off" class="form-control requiredField validate-required" maxlength="3"/>';
            echo '</div>';
            echo '';
            echo '<div class="col-sm-12"><br/></div>';
            echo '';
            echo '<div class="col-sm-12">';
            echo '<label class="control-label">' . __('Card Holder Name', 'woocommerce-smartroute-direct-post-gateway') . ' <span style="color:red">*</span> </label>';
            echo '</div>';
            echo '<div class="col-sm-12">';
            echo '<input type="text" name="SrdpCardHolderName" value="" class="form-control requiredField validate-required" autocomplete="off" data-validate="required" maxlength="64"/>';
            echo '<span class="error_element"></span>';
            echo '</div>';
            echo '</div>';
            echo '';
            echo '<div class="clearfix">&nbsp;</div>';
            echo '<div class="col-xs-12"><br/></div>';
            echo '';
            echo '</div>';
        }
        if (in_array('sadad', $this->MethodType)) {
            echo '<div class="col-sm-12">';
            echo '<div class="col-sm-12 noPadding">';
            echo '<ul class="wc_payment_methods payment_methods methods"><li>'
            . '<input type="radio" id="SrdpPaymentMethod-sadad" autocomplete="off" name="SrdpPaymentMethod" value="2"/>'
            . '<label for="SrdpPaymentMethod-sadad"><img class="sadad_image" src="' . WP_PLUGIN_URL . DIRECTORY_SEPARATOR . Smartroute_Direct_Post::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'sadadLogo_low_en.png' . '" width="100"height="25"></label>'
            . '</li></ul>';
            echo '</div>';
            echo '';
            echo '<div data-payment-method="2">';
            echo '<div class="col-sm-12">';
            echo '<label class="control-label">' . __('SADAD OLP ID', 'woocommerce-smartroute-direct-post-gateway') . ' <span style="color:red">*</span> </label>';
            echo '</div>';
            echo '<div class="col-sm-12">';
            echo '<div class="col-sm-12 noPadding">';
            echo '<input type="text" name="SrdpSadadOlpId" data-validate="required" autocomplete="off" value="" class="form-control sadadRequiredField "/>';
            echo '</div>';
            echo '</div>';
            echo '</div>';
        }
        echo '';
        echo '<div class="col-sm-12"><br/></div>';
        echo '';
        echo '</div>';
        echo '</div>';
        echo '</div>';
        echo '</div>';
        $invalidCardNumberError = __('Invalid Card Number', 'woocommerce-smartroute-direct-post-gateway');
        $fieldRequiredErrorMsg = __('Field is required', 'woocommerce-smartroute-direct-post-gateway');
        $invalidSecurityCodeErrorMsg = __('Invalid Security Code', 'woocommerce-smartroute-direct-post-gateway');
        echo <<<OEF
<script>
    var rad = document.querySelectorAll('input[name="SrdpPaymentMethod"]');    
    var prev = null;
    for(var i = 0; i < rad.length; i++) {
        rad[i].onclick = function() {
            (prev)? console.log('prev ... '+prev.value):null;
            if(this !== prev) {
                prev = this;
            }
            
            var current_inputs = document.querySelectorAll('[data-payment-method="'+this.value+'"] input, [data-payment-method="'+this.value+'"] select');            
            for(jc in current_inputs){
                current_inputs[jc].disabled = false;
            }
            
            var other_value;
            if(this.value==1){
                other_value = 2;                
            }else{
                other_value = 1;
            }
            var prev_elem = document.querySelectorAll('[data-payment-method="'+other_value+'"]');                
            var prev_inputs = document.querySelectorAll('[data-payment-method="'+other_value+'"] input, [data-payment-method="'+other_value+'"] select');            
            for(jv in prev_inputs){
                prev_inputs[jv].disabled = 'disabled';
                error_fields = document.querySelectorAll('#error'+prev_inputs[jv].name);
                if(error_fields.length==1){
                    elem = error_fields[0];
                    elem.parentNode.removeChild(elem);                    
                    prev_inputs[jv].classList.remove("woocommerce-invalid");                    
                }                
                if(prev_inputs[jv].type !== 'hidden'){
                    prev_inputs[jv].value = '';
                }        
            }
        };
    }
    rad[0].click();
    
    function validateDirectPostForm(){
        var form_fields = document.querySelectorAll('input[type="text"], input[type="password"], select');
        var has_errors = false;
        for(i=0; i<form_fields.length; i++){
            field = form_fields[i];

            validation_rule = field.dataset?field.dataset.validate:'';            
            is_error = false;
            error_msg = "";           
            if(validation_rule && field.disabled === false){
               var validations = validation_rule.split("|");
                for(j=0;j<validations.length;j++){
                    validation_rule = validations[j];
                    
                    if(validation_rule == 'required' && field.value==""){                    
                        is_error = true;
                        error_msg = "{$fieldRequiredErrorMsg}";
                    }else if(validation_rule=='creditcard' && !validCreditCard(field.value)){
                        is_error = true;
                        error_msg = "{$invalidCardNumberError}";
                    }else if(validation_rule=='cvv'){
                        if(field.value==""){
                            is_error = true;
                            error_msg = "{$fieldRequiredErrorMsg}";   
                        }else if(!validCVV(field.value)){
                            is_error = true;
                            error_msg = "{$invalidSecurityCodeErrorMsg}";
                        }
                    }
                }
                
                has_errors = has_errors || is_error;
            }

            if(is_error){
                if(!field.classList.contains('required-entry')){
                    field.className += ' required-entry';   
                }                
                if(!field.classList.contains('woocommerce-invalid')){
                    field.className += ' woocommerce-invalid';   
                }
                error_fields = document.querySelectorAll('span#error'+field.name);        
                if(error_fields.length>0){
                    var error_el = error_fields[0];
                }else{
                    var error_el = document.createElement("span");
                }
                error_el.className='ywccp_error';                
                error_el.innerHTML = error_msg;
                field.className += ' woocommerce-invalid';
                error_el.setAttribute("for",field.name);
                error_el.setAttribute("id",'error'+field.name);
                if(error_fields.length == 0){
                    insertAfter(error_el, field);
                }
            }else{
                error_fields = document.querySelectorAll('#error'+field.name);
                if(error_fields.length==1){
                    elem = error_fields[0];
                    elem.parentNode.removeChild(elem);
                }
                field.classList.remove("woocommerce-invalid");
                field.classList.remove("required-entry");
            }
        }
        
        return !has_errors;
    }
        
    document.forms['checkout'].onsubmit = function(submitEvent){
        if (validateDirectPostForm() == false){                    
            submitEvent.preventDefault();        
            return false;
        }
    }; 
    
    function insertAfter(newNode, referenceNode) {               
        console.log('inserting after '+referenceNode.name+ ' error field '+newNode.outerHTML);
        jQuery(newNode.outerHTML).insertAfter('[name="'+referenceNode.name+'"]');
    }
    
    // takes the form field value and returns true on valid number
    function validCreditCard(value) {
    
	if (/[^0-9-\s]+/.test(value)) return false;
	// The Luhn Algorithm. It's so pretty.
	var nCheck = 0, nDigit = 0, bEven = false;
	value = value.replace(/\D/g, "");

	for (var n = value.length - 1; n >= 0; n--) {
		var cDigit = value.charAt(n),
			  nDigit = parseInt(cDigit, 10);

		if (bEven) {
			if ((nDigit *= 2) > 9) nDigit -= 9;
		}

		nCheck += nDigit;
		bEven = !bEven;
	}

	return (nCheck % 10) == 0;
    }    
    
    function validCVV(value){        
        ret = value.search(/^[0-9]{3,4}$/);                
        return ret==0;
    }
</script>             
OEF
        ;
    }

    // generate the secure hash
    private function generateDirectPostSecureHash($array) {
        $authenticationToken = trim($this->auth_token);
        ksort($array);
        $orderedString = $authenticationToken;

        // encoding parameters 
        $encoding_parameters = array();
        $encoding_parameters[] = "Response.StatusDescription";
        $encoding_parameters[] = "Response.GatewayStatusDescription";
        $encoding_parameters[] = "PARes";

        foreach ($array as $k => $param) {
            if ($param !== 'null') {
                if (in_array($k, $encoding_parameters)) {
                    $orderedString .= urlencode($param);
                } else {
                    $orderedString .= $param;
                }
            }
        }

        $secureHash = hash('sha256', $orderedString, false);

        return $secureHash;
    }

    private function checkIfNull($value) {
        if ($value == null || $value == "null" || $value == '') {
            return null;
        } else {
            return $value;
        }
    }

    public function log($message, $level = 'debug') {
        if ($this->enableLog) {
            if (empty(self::$log)) {
                self::$log = new WC_Logger();
            }

            self::$log->add('smartroute-directpost', $message, $level);
        }
    }

    public function inquirySingleOrder(WC_Order &$order) {

        $inq_url = trim($this->InquiryURL);

        $fields = array();
        $fields['MessageID'] = 2;
        $fields['MerchantID'] = trim($this->MerchantID);
        $fields['OriginalTransactionID'] = $order->get_transaction_id();
        $secureHash = trim($this->auth_token);
        $fields['SecureHash'] = SmartRoute_Direct_Post_Helper::generateDPSecureHash($fields, $secureHash);

        $this->log('calling inquiry for order#' . $order->get_transaction_id() . ' parameters: ' . json_encode($fields));
        $inq_response = wp_remote_post($inq_url, array(
            'method' => 'POST',
            'body' => http_build_query($fields),
            'timeout' => 90,
            'sslverify' => false,
        ));


        // Retrieve the body's resopnse if no errors found
        $inq_response_body = wp_remote_retrieve_body($inq_response);

        // Parse the response into something we can read
        $inq_resp = array();
        parse_str($inq_response_body, $inq_resp);
        $this->log('inquiry for order#' . $order->get_transaction_id() . ' response: ' . $inq_response_body);


        //get required params from response
        $inq_res_data = array_map(function($item) {
            return SmartRoute_Direct_Post_Helper::checkIfNull($item);
        }, $inq_resp);
        unset($inq_res_data['Response_SecureHash']);


        // get the secure hash
        $res_secure_hash = SmartRoute_Direct_Post_Helper::checkIfNull($inq_resp['Response_SecureHash']);
        $newSecureHash = SmartRoute_Direct_Post_Helper::generateDPSecureHash($inq_res_data, trim($this->auth_token));


        if ($newSecureHash == $res_secure_hash) {
            $this->log('inquiry for order#' . $order->get_transaction_id() . ' secure hash match, status: ' . $inq_res_data['Response_StatusCode']);
            if ($inq_res_data['Response_StatusCode'] == "00000") {
                $order->update_status('wc-completed', 'order_note');
            } else {
                $order->update_status('wc-failed', 'order_note');
            }
        } else {
            // do nothing
            $this->log('inquiry for order#' . $order->get_transaction_id() . ' secure hash not match', 'error');
        }
    }

}

/// End of Smartroute_Direct_Post


