<?php

// This URL will be  used  in 'woocomerce-smartroute-redirect-response.php' to inquier about the transaction when secure hash not match
// define('invalidSecureHashInquiryURL', '', true);

/* SmartRoute Redirect Payment Gateway Class */
require_once 'woocommerce-smartroute-redirect-helper.php';

class SmartRoute_Redirect extends WC_Payment_Gateway {

    public static $PLUGIN_FOLDER_NAME = 'woocommerce-smartroute-redirect-gateway';
    private static $log;
    public static $invalidSecureHashInquiryURL;

    // Setup our Gateway's id, description and other values
    function __construct() {
        // The global ID for this Payment method
        $this->id = "smartroute_redirect";

        // The Title shown on the top of the Payment Gateways Page next to all the other Payment Gateways
        $this->method_title = __("SmartRoute Redirect Payment", 'smartroute-redirect');

        $this->supports[] = 'refunds';

        // The description for this Payment Gateway, shown on the actual Payment options page on the backend
        $this->method_description = __("SmartRoute Redirect Payment Gateway Plug-in for WooCommerce", 'smartroute-redirect');

        // The title to be used for the vertical tabs that can be ordered top to bottom
        $this->title = __("SmartRoute Redirect Payment", 'smartroute-redirect');

        // If you want to show an image next to the gateway's name on the frontend, enter a URL to an image.
        $this->icon = WP_PLUGIN_URL . DIRECTORY_SEPARATOR . SmartRoute_Redirect::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'SmartRoute.png';


        // Bool. Can be set to true if you want payment fields to show on the checkout 
        // if doing a direct integration, which we are doing in this case
        $this->has_fields = true;

        // This basically defines your settings which are then loaded with init_settings()
        $this->init_form_fields();

        // After init_settings() is called, you can get the settings and load them into variables, e.g:
        // $this->title = $this->get_option( 'title' );
        $this->init_settings();

        // Turn these settings into variables we can use
        foreach ($this->settings as $setting_key => $value) {
            $this->$setting_key = $value;
        }

        // Lets check for SSL
        add_action('admin_notices', array($this, 'do_ssl_check'));


        // Save settings
        if (is_admin()) {
            // Versions over 2.0
            // Save our administration options. Since we are not going to be doing anything special
            // we have not defined 'process_admin_options' in this class so the method in the parent
            // class will be used instead
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        }

        SmartRoute_Redirect::$invalidSecureHashInquiryURL = $this->InquiryURL;
    }

    // End __construct()
    // Build the administration fields for this specific Gateway
    public function init_form_fields() {
        $this->form_fields = array(
            'enabled' => array(
                'title' => __('Enable / Disable', 'smartroute-redirect'),
                'label' => __('Enable this payment gateway', 'smartroute-redirect'),
                'type' => 'checkbox',
                'default' => 'no',
            ),
            'title' => array(
                'title' => __('Title', 'smartroute-redirect'),
                'type' => 'text',
                'desc_tip' => __('Payment title the customer will see during the checkout process.', 'smartroute-redirect'),
                'default' => __('SmartRoute Redirect Payment', 'smartroute-redirect'),
            ),
            'description' => array(
                'title' => __('Description', 'smartroute-redirect'),
                'type' => 'textarea',
                'desc_tip' => __('Payment description the customer will see during the checkout process.', 'smartroute-redirect'),
                'default' => __('Pay securely using your Credit card or SADAD account.', 'smartroute-redirect'),
                'css' => 'max-width:350px;'
            ),
            'MerchantID' => array(
                'title' => __('Merchant ID', 'smartroute-redirect'),
                'type' => 'text',
                'desc_tip' => __('This is the Merchant ID.', 'smartroute-redirect'),
            ),
            'auth_token' => array(
                'title' => __('Authentication Token', 'smartroute-redirect'),
                'type' => 'text',
                'desc_tip' => __('This is the Authentication Token provided by SmartRoute Redirect Payment for the merchant.', 'smartroute-redirect'),
            ),
            'ItemID' => array(
                'title' => __('Item ID', 'smartroute-redirect'),
                'type' => 'text',
                'desc_tip' => __('This is the custom Item ID for the merchant.', 'smartroute-redirect'),
            ),
            'RedirectPaymentURL' => array(
                'title' => __('Redirect Payment URL', 'smartroute-redirect'),
                'type' => 'text',
                'desc_tip' => __('This is the payment URL for smartroute redirect.', 'smartroute-redirect'),
            ),
            'RefundURL' => array(
                'title' => __('Refund URL', 'smartroute-redirect'),
                'type' => 'text',
                'desc_tip' => __('This is the refund URL for smartroute redirect.', 'smartroute-redirect'),
            ),
            'InquiryURL' => array(
                'title' => __('Inquiry URL', 'smartroute-redirect'),
                'type' => 'text',
                'desc_tip' => __('This is the Inquiry URL for smartroute redirect.', 'smartroute-redirect'),
            ),
            'enableLog' => array(
                'title' => __('Enable Logs', 'woocommerce-smartroute-direct-post-gateway'),
                'label' => __('Enable logs for this payment gateway', 'woocommerce-smartroute-direct-post-gateway'),
                'type' => 'checkbox',
                'default' => 'no',
            ),
        );
    }

    // Submit payment and handle response
    public function process_payment($order_id) {
        global $woocommerce;

        // Get this Order's information so that we know
        // who to charge and how much
        $order = new WC_Order($order_id);

        // Decide which URL to post to
        $currencies_file = parse_ini_file("currencies.ini");

        // This is where the fun stuff begins
        $data = array();
        $data['MessageID'] = 1;
        $data['TransactionID'] = (int) (microtime(true) * 1000);
        $data['MerchantID'] = trim($this->MerchantID);
        if (strpos(get_locale(), '_') !== false) {
            $data['Language'] = substr(get_locale(), 0, strpos(get_locale(), '_'));
        } else {
            $data['Language'] = get_locale();
        }
        
	$this->log('Processing payment for order #' . $order_id . ', and transaction ID ' . $data['TransactionID'] . '.');

        $old_wc = version_compare(WC_VERSION, '3.0', '<');
        $order_total = $old_wc ? $order->order_total : $order->get_total();
        $data['Amount'] = str_replace('.', '', $order_total);
        $data['CurrencyISOCode'] = $currencies_file[get_woocommerce_currency()];

        if (strpos(wc_get_page_permalink('checkout'), '?') !== false) {
            $data['ResponseBackURL'] = wc_get_page_permalink('checkout') . '&handleRedirectPayment'; //'/checkout-2/?handleRedirectPayment';
        } else {
            $data['ResponseBackURL'] = wc_get_page_permalink('checkout') . '?handleRedirectPayment'; //'/checkout-2/?handleRedirectPayment';
        }
        if (trim($this->ItemID) != '') {
            $data['ItemID'] = trim($this->ItemID);
        }
        $data['SecureHash'] = $this->generateRedirectSecureHash($data);
        $this->log("redirect parameters order#" . $order_id . ': ' . json_encode($data), 'debug');

        $RedirectPaymentURL = trim($this->RedirectPaymentURL);
        $order->set_transaction_id($data['TransactionID']);
        $order->save();
        $order->update_status('wc-pending', 'order_note'); //revert payment to pending
        //build query string
        $query_string = "";
        $first_param = true;
        foreach ($data as $key => $value) {
            if ($first_param) {
                $query_string .= $key . '=' . $value;
                $first_param = false;
            } else {
                $query_string .= '&' . $key . '=' . $value;
            }
        }

        //set order id in the session
        session_start();
        $_SESSION["auth_token"] = $this->auth_token;
        $_SESSION["order_id"] = $order_id;

	$this->log('Transaction sent to Smartroute');
        return array(
            'result' => 'success',
            'redirect' => $RedirectPaymentURL,
            'params' => $query_string,
            'redirect_type' => 'post'
        );
    }

    public function process_refund($order_id, $amount = null, $reason = '') {

        $order = wc_get_order($order_id);
        if (!$order) {
            return false;
        }
        $this->log("Beginning refund for order $order_id for the amount of {$amount}", 'info');

        $currencies_file = parse_ini_file("currencies.ini");

        $url = trim($this->RefundURL);


        if ($order->post_status == 'wc-failed') {
            return new WP_Error('smartroute_error', __('Order is not suitable for refund.', 'smartroute-redirect'));
        }

        // check if the amount is empty
        if ($amount == null || $amount == "null" || $amount == '' || $amount == 0) {
            return new WP_Error('smartroute_error', __('The refund amount should not be empyt or 0.', 'smartroute-redirect'));
        }

        // check if the amount less than or equal zero
        if ($amount < 0) {
            return new WP_Error('smartroute_error', __('Invalid refund amount.', 'smartroute-redirect'));
        }

        //prepare the request parameters
        $transactionID = "wcref" . time() . rand(1000, 9999);
        $fields = array();
        $fields['MessageID'] = 4;
        $fields['MerchantID'] = trim($this->MerchantID);
        $fields['TransactionID'] = $transactionID;
        $fields['OriginalTransactionID'] = $order->get_transaction_id();
        //$fields['OriginalTransactionID'] = str_replace("order", "", str_replace("_","",$order->order_key));
        $fields['CurrencyISOCode'] = $currencies_file[get_woocommerce_currency()];
        $fields['Amount'] = str_replace('.', '', $amount);
        $fields['SecureHash'] = $this->generateRedirectSecureHash($fields);

        $response = wp_remote_post($url, array(
            'method' => 'POST',
            'body' => http_build_query($fields),
            'timeout' => 90,
            'sslverify' => false,
        ));



        // Retrieve the body's resopnse if no errors found
        $response_body = wp_remote_retrieve_body($response);

        $this->log("refund step 1 successfully for order $order_id for the amount of {$amount}", 'info');

        // Parse the response into something we can read
        $result = array();
        foreach (explode("&", $response_body) as $parameter) {
            $equalsIndex = strpos($parameter, "=");
            $name = substr($parameter, 0, $equalsIndex);
            $value = substr($parameter, $equalsIndex + 1);

            $result[$name] = $value;
        }

        $res_data = array();

        //get required params from response
        if (isset($result['Response.OriginalTransactionID'])) {
            $res_data['Response.OriginalTransactionID'] = $this->checkIfNull($result['Response.OriginalTransactionID']);
        }

        if (isset($result['Response.StatusCode'])) {
            $res_data['Response.StatusCode'] = $this->checkIfNull($result['Response.StatusCode']);
        }

        if (isset($result['Response.StatusDescription'])) {
            $res_data['Response.StatusDescription'] = $this->checkIfNull($result['Response.StatusDescription']);
        }

        if (isset($result['Response.Amount'])) {
            $res_data['Response.Amount'] = $this->checkIfNull($result['Response.Amount']);
        }

        if (isset($result['Response.CurrencyISOCode'])) {
            $res_data['Response.CurrencyISOCode'] = $this->checkIfNull($result['Response.CurrencyISOCode']);
        }

        if (isset($result['Response.MerchantID'])) {
            $res_data['Response.MerchantID'] = $this->checkIfNull($result['Response.MerchantID']);
        }

        if (isset($result['Response.TransactionID'])) {
            $res_data['Response.TransactionID'] = $this->checkIfNull($result['Response.TransactionID']);
        }

        if (isset($result['Response.MessageID'])) {
            $res_data['Response.MessageID'] = $this->checkIfNull($result['Response.MessageID']);
        }

        //get optional params from response
        if (isset($result['Response.RRN'])) {
            $res_data['Response.RRN'] = $this->checkIfNull($result['Response.RRN']);
        }

        // get the secure hash
        $success = false;
        $is_inq = false;
        $res_secure_hash = $this->checkIfNull($result['Response.SecureHash']);
        $newSecureHash = $this->generateRedirectSecureHash($res_data);

        $status = (isset($res_data['Response.StatusCode']) AND ! empty($res_data['Response.StatusCode'])) ? $res_data['Response.StatusCode'] : NULL;


        //test the securehash
        if ($status != NULL AND $newSecureHash == $res_secure_hash) {
            if ($res_data['Response.StatusCode'] == "00000") {
                $success = true;
                if ($order->post_status == 'wc-pending') {
                    $order->update_status('wc-completed', 'order_note');
                }

                // status code not success and not abnormal error
            } else if ($res_data['Response.StatusCode'] != "00019") {
                $order->update_status('wc-failed', 'order_note');
                return new WP_Error('smartroute_error', __('Order is not suitable for refund.', 'smartroute-redirect'));
            }

            $this->log("refund status code 00000 successfully for order $order_id for the amount of {$amount}", 'info');
        } else {
            //Inquiry Message
            $is_inq = true;
            $inq_url = trim($this->InquiryURL);

            $fields = array();
            $fields['MessageID'] = 2;
            $fields['MerchantID'] = trim($this->MerchantID);
            $fields['OriginalTransactionID'] = $transactionID;
            $fields['SecureHash'] = $this->generateRedirectSecureHash($fields);

            $this->log("refund inquiry message for order {$order_id} parameter " . http_build_query($fields), 'info');
            $inq_response = wp_remote_post($inq_url, array(
                'method' => 'POST',
                'body' => http_build_query($fields),
                'timeout' => 90,
                'sslverify' => false,
            ));

            // Retrieve the body's resopnse if no errors found
            $inq_response_body = wp_remote_retrieve_body($inq_response);
            $this->log("refund inquiry message for order {$order_id} response " . $inq_response_body, 'info');
            // Parse the response into something we can read
            $inq_resp = array();
            foreach (explode("&", $inq_response_body) as $parameter) {
                $equalsIndex = strpos($parameter, "=");
                $name = substr($parameter, 0, $equalsIndex);
                $value = substr($parameter, $equalsIndex + 1);

                $inq_resp[$name] = $value;
            }
            $inq_res_data = array();
            //get required params from response
            if (isset($inq_resp['Response.MessageStatus'])) {
                $inq_res_data['Response.MessageStatus'] = $this->checkIfNull($inq_resp['Response.MessageStatus']);
            }

            if (isset($inq_resp['Response.StatusCode'])) {
                $inq_res_data['Response.StatusCode'] = $this->checkIfNull($inq_resp['Response.StatusCode']);
            }

            if (isset($inq_resp['Response.StatusDescription'])) {
                $inq_res_data['Response.StatusDescription'] = $this->checkIfNull($inq_resp['Response.StatusDescription']);
            }

            if (isset($inq_resp['Response.Amount'])) {
                $inq_res_data['Response.Amount'] = $this->checkIfNull($inq_resp['Response.Amount']);
            }

            if (isset($inq_resp['Response.CurrencyISOCode'])) {
                $inq_res_data['Response.CurrencyISOCode'] = $this->checkIfNull($inq_resp['Response.CurrencyISOCode']);
            }

            if (isset($inq_resp['Response.MerchantID'])) {
                $inq_res_data['Response.MerchantID'] = $this->checkIfNull($inq_resp['Response.MerchantID']);
            }

            if (isset($inq_resp['Response.TransactionID'])) {
                $inq_res_data['Response.TransactionID'] = $this->checkIfNull($inq_resp['Response.TransactionID']);
            }

            if (isset($inq_resp['Response.MessageID'])) {
                $inq_res_data['Response.MessageID'] = $this->checkIfNull($inq_resp['Response.MessageID']);
            }

            if (isset($inq_resp['Response.ReversalStatus'])) {
                $inq_res_data['Response.ReversalStatus'] = $this->checkIfNull($inq_resp['Response.ReversalStatus']);
            }

            //get optional params from response
            if (isset($inq_resp['Response.GatewayStatusCode'])) {
                $inq_res_data['Response.GatewayStatusCode'] = $this->checkIfNull($inq_resp['Response.GatewayStatusCode']);
            }

            if (isset($inq_resp['Response.GatewayStatusDescription'])) {
                $inq_res_data['Response.GatewayStatusDescription'] = $this->checkIfNull($inq_resp['Response.GatewayStatusDescription']);
            }

            if (isset($inq_resp['Response.GatewayName'])) {
                $inq_res_data['Response.GatewayName'] = $this->checkIfNull($inq_resp['Response.GatewayName']);
            }

            if (isset($inq_resp['Response.RRN'])) {
                $inq_res_data['Response.RRN'] = $this->checkIfNull($inq_resp['Response.RRN']);
            }

            if (isset($inq_resp['Response.ApprovalCode'])) {
                $inq_res_data['Response.ApprovalCode'] = $this->checkIfNull($inq_resp['Response.ApprovalCode']);
            }

            if (isset($inq_resp['Response.CardExpiryDate'])) {
                $inq_res_data['Response.CardExpiryDate'] = $this->checkIfNull($inq_resp['Response.CardExpiryDate']);
            }

            if (isset($inq_resp['Response.CardHolderName'])) {
                $inq_res_data['Response.CardHolderName'] = $this->checkIfNull($inq_resp['Response.CardHolderName']);
            }

            if (isset($inq_resp['Response.CardNumber'])) {
                $inq_res_data['Response.CardNumber'] = $this->checkIfNull($inq_resp['Response.CardNumber']);
            }

            // get the secure hash
            $res_secure_hash = $this->checkIfNull($inq_resp['Response.SecureHash']);
            $newSecureHash = $this->generateRedirectSecureHash($inq_res_data);

            if ($newSecureHash == $res_secure_hash) {
                if ($inq_res_data['Response.StatusCode'] == "00000") {
                    $this->log("refund enqury code 00000 successfully for order $order_id for the amount of {$amount}", 'info');
                    $success = true;
                    if ($order->post_status == 'wc-pending') {
                        $order->update_status('wc-completed', 'order_note');
                    }
                    // status code not success and not abnormal error	
                } else if ($res_data['Response.StatusCode'] != "00019") {
                    $this->log("refund enqury code 00019  for order $order_id for the amount of {$amount}", 'info');
                    $order->update_status('wc-failed', 'order_note');
                    return new WP_Error('smartroute_error', __('Order is not suitable for refund.', 'smartroute-redirect'));
                }
            } else {
                // do nothing
            }
        }

        if ($success) {
            if ($is_inq) {
                $refund_message = sprintf(__('Refunded %1$s - Refund ID: %2$s - Reason: %3$s', 'smartroute-redirect'), $amount, $inq_res_data['Response.TransactionID'], $reason);
            } else {
                $refund_message = sprintf(__('Refunded %1$s - Refund ID: %2$s - Reason: %3$s', 'smartroute-redirect'), $amount, $res_data['Response.TransactionID'], $reason);
            }
            $order->add_order_note($refund_message);
            $this->log('Success: ' . html_entity_decode(strip_tags($refund_message)));
            return true;
        } else {
            $this->log('Error: ' . $res_data['Response.StatusDescription']);
            return new WP_Error('smartroute_error', __('There was a problem connecting to the payment gateway.', 'smartroute-redirect'));
        }
    }

    // generate the secure hash
    private function generateRedirectSecureHash($array) {
        $authenticationToken = trim($this->auth_token);
        ksort($array);
        $orderedString = $authenticationToken;

        // encoding parameters 
        $encoding_parameters = array();
        $encoding_parameters[] = "Response.StatusDescription";
        $encoding_parameters[] = "Response.GatewayStatusDescription";
        $encoding_parameters[] = "Response_StatusDescription";
        $encoding_parameters[] = "Response_GatewayStatusDescription";

        foreach ($array as $k => $param) {
            if ($param !== 'null') {
                if (in_array($k, $encoding_parameters)) {
                    $orderedString .= urlencode($param);
                } else {
                    $orderedString .= $param;
                }
            }
        }
        //error_log("ordered string ".$orderedString);
        $secureHash = hash('sha256', $orderedString, false);

        return $secureHash;
    }

    /**
     * Format prices.
     * @param  float|int $price
     * @param  WC_Order $order
     * @return string
     */
    protected function number_format($price, $order) {
        $decimals = 2;

        if (!$this->currency_has_decimals($order->get_currency())) {
            $decimals = 0;
        }

        return number_format($price, $decimals, '.', '');
    }

    /**
     * Check if currency has decimals.
     * @param  string $currency
     * @return bool
     */
    protected function currency_has_decimals($currency) {
        if (in_array($currency, array('HUF', 'JPY', 'TWD'))) {
            return false;
        }

        return true;
    }

    // Check if we are forcing SSL on checkout pages
    // Custom function not required by the Gateway
    public function do_ssl_check() {

        if ($this->enabled == "yes") {
            if (get_option('woocommerce_force_ssl_checkout') == "no") {
                echo "<div class=\"error\"><p>" . sprintf(__("<strong>%s</strong> is enabled and WooCommerce is not forcing the SSL certificate on your checkout page. Please ensure that you have a valid SSL certificate and that you are <a href=\"%s\">forcing the checkout pages to be secured.</a>"), $this->method_title, admin_url('admin.php?page=wc-settings&tab=checkout')) . "</p></div>";
            }
        }
    }

    //handle order status
    private function getOrderStatusId($status_code) {
        switch ($status_code) {
            case '10008':
                return "cancelled";

            case '00000':
                return "completed";

            case '20001':
                return "pending";

            case '20002':
                return "pending";

            default:
                return "failed";
        }
    }

    private function checkIfNull($value) {
        if ($value == null || $value == "null") {
            return null;
        } else {
            return $value;
        }
    }

    public function log($message, $level = 'debug') {
        if ($this->enableLog) {
            if (empty(self::$log)) {
                self::$log = new WC_Logger();
            }

            self::$log->add('smartroute-redirect', $message, $level);
            error_log($message);
        }
    }

    public function inquirySingleOrder(WC_Order &$order) {

        $inq_url = trim($this->InquiryURL);

        $fields = array();
        $fields['MessageID'] = 2;
        $fields['MerchantID'] = trim($this->MerchantID);
        $fields['OriginalTransactionID'] = $order->get_transaction_id();
        $secureHash = trim($this->auth_token);
        $fields['SecureHash'] = SmartRoute_Redirect_Helper::generateRedirectHandlerSecureHash($fields, $secureHash);

        $this->log('smartroute inquiry trans#' . $order->get_transaction_id() . ' parameters: ' . json_encode($fields));
        $inq_response = wp_remote_post($inq_url, array(
            'method' => 'POST',
            'body' => http_build_query($fields),
            'timeout' => 90,
            'sslverify' => false,
        ));



        // Retrieve the body's resopnse if no errors found
        $inq_response_body = wp_remote_retrieve_body($inq_response);
        $this->log('smartroute inquiry trans#' . $order->get_transaction_id() . ' response: ' . $inq_response_body);
        // Parse the response into something we can read
        $inq_resp = array();
        parse_str($inq_response_body, $inq_resp);

        //get required params from response
        $inq_res_data = array_map(function($item) {
            return SmartRoute_Redirect_Helper::checkIfNull($item);
        }, $inq_resp);
        unset($inq_res_data['Response_SecureHash']);



        // get the secure hash
        $res_secure_hash = SmartRoute_Redirect_Helper::checkIfNull($inq_resp['Response_SecureHash']);
        $newSecureHash = SmartRoute_Redirect_Helper::generateRedirectHandlerSecureHash($inq_res_data, trim($this->auth_token));

        if ($newSecureHash == $res_secure_hash) {
            $this->log('smartroute inquiry trans#' . $order->get_id() . ' status code=' . $inq_res_data['Response_StatusCode']);
            if ($inq_res_data['Response_StatusCode'] == "00000") {
                $order->update_status('wc-completed', 'order_note');
            } else {
                $order->update_status('wc-failed', 'order_note');
            }
        } else {
            // do nothing
            $this->log('smartroute inquiry trans#' . $order->get_id() . ' secure hash not match', 'error');
        }
    }

}

// End of SmartRoute_Redirect
