<?php

/*
  Plugin Name: SmartRoute Redirect - WooCommerce Gateway
  Plugin URI: http://www.stspayone.com/
  Description: Extends WooCommerce by Adding the SmartRoute Redirect Gateway.
  Version: 1.0.4
  Author: STS PayOne
  Author URI: http://www.stspayone.com/
 */

// Include our Gateway Class and register Payment Gateway with WooCommerce
add_action('plugins_loaded', 'smartroute_redirect_init', 0);
add_action('plugins_loaded', 'schedule_redirect_inquiry_operations', 2);
add_action('plugins_loaded', 'redirect_load_textdomain', 1);

function smartroute_redirect_init() {
    // If the parent WC_Payment_Gateway class doesn't exist
    // it means WooCommerce is not installed on the site
    // so do nothing
    if (!class_exists('WC_Payment_Gateway'))
        return;

    // If we made it this far, then include our Gateway Class
    include_once( 'woocommerce-smartroute-redirect.php' );

    // Now that we have successfully included our class,
    // Lets add it too WooCommerce
    add_filter('woocommerce_payment_gateways', 'add_smartroute_redirect_gateway');

    function add_smartroute_redirect_gateway($methods) {
        $methods[] = 'SmartRoute_Redirect';
        return $methods;
    }

}

// Add custom action links
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'smartroute_redirect_action_links');

function smartroute_redirect_action_links($links) {
    $plugin_links = array(
        '<a href="' . admin_url('admin.php?page=wc-settings&tab=checkout') . '">' . __('Settings', 'smartroute-redirect') . '</a>',
    );

    // Merge our new link with the default ones
    return array_merge($plugin_links, $links);
}

add_action('plugins_loaded', 'smartroute_redirect_run');

/**
 * Run the plugin.
 */
function smartroute_redirect_run() {
    require_once('functions.php' );
}

add_action('init', 'add_smartroute_redirect_endpoint');

function add_smartroute_redirect_endpoint() {
    add_rewrite_endpoint('handleRedirectPayment', EP_ROOT | EP_PAGES);
}

add_action('woocommerce_before_checkout_form_cart_notices', 'include_payment_details', 10, 5);

function include_payment_details() {
    global $wp_query;
    if (isset($wp_query->query['handleRedirectPayment']) &&
        (!isset($_SESSION['smartRouteEntered']) || !$_SESSION['smartRouteEntered'])) {
        load_template(WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . SmartRoute_Redirect::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'woocommerce-smartroute-redirect-response.php');
    }
    return true;
}

/*
This is how t he above function looked like when STS received this plug in
The below method replaced each of the searchbox, breadcrumbs,and reattempt payment views with the payment details
The new function includes the payment details without touching the original page structure

add_filter('wc_get_template', 'custom_direct_post_endpoint', 10, 5);

function custom_direct_post_endpoint($located, $template_name, $args, $template_path, $default_path) {
    global $wp_query;

    if (isset($wp_query->query['handleDpResponse']) &&
            (!isset($_SESSION['smartRouteDirectPostEntered']) || !$_SESSION['smartRouteDirectPostEntered']) ) {

echo "<script>console.log('" . $template_name . "')</script>";

        $located = WP_PLUGIN_DIR . DIRECTORY_SEPARATOR . SmartRoute_Direct_Post::$PLUGIN_FOLDER_NAME . DIRECTORY_SEPARATOR . 'woocommerce-smartroute-direct-post-response.php';
    }

    return $located;
}
*/

function redirect_load_textdomain() {
    load_plugin_textdomain('woocommerce-smartroute-redirect-gateway', false, dirname(plugin_basename(__FILE__)) . '/i18n/languages/');
}

function schedule_redirect_inquiry_operations() {
    add_filter('cron_schedules', 'callback_add_inquiry_redirect_schedule');

    add_action(
            'smartroute_redirect_inquiry_cron_hook', 'smartroute_redirect_inquiry_hook'
    );

    if (!wp_next_scheduled('smartroute_redirect_inquiry_cron_hook')) {
        wp_schedule_event(
                time(), 'redirect-inquiry-interval', 'smartroute_redirect_inquiry_cron_hook'
        );
    }

    return;
}

function callback_add_inquiry_redirect_schedule($schedules) {
    $schedules['redirect-inquiry-interval'] = array(
        'interval' => 5 * 60, //every 5 minutes
        'display' => __('Redirect Inquiry Interval (5 min)', 'woocommerce-smartroute-redirect-gateway'),
    );
    return $schedules;
}

function smartroute_redirect_inquiry_hook() {
    //error_log("scheduled job " . __METHOD__ . ' is called...');

    $args = array(        
        'status' => array('wc-pending', 'wc-processing'),
        'payment_method' => 'smartroute_redirect',
        'created_via' => 'checkout',
        'type' => 'shop_order',
        'date_before' => gmdate('Y-m-d H:i:s', time() - 5 * 60),
        'limit' => -1,
    );
    $orders = wc_get_orders($args);
    
    $gateway = new SmartRoute_Redirect();
    
    foreach($orders as $order){
        if($order->get_transaction_id()){
            $gateway->log('calling inquiry for order trans ID#'.$order->get_id());
            $gateway->inquirySingleOrder($order);            	    
        }else{
	       $gateway->log('cannot find transaction id for order#'.$order->get_id().' to inquire','error');
	    }
    }    
}

